﻿using Microsoft.AspNetCore.Mvc;
using Model;
using StubLib;
using DTO;
using DTO.Mapper;
using System.CodeDom.Compiler;
using System.Drawing;
using System;
using API_LoL.Mapper;
using System.Xml.Linq;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace API_LoL.Controllers
{
    //[Route("api/[controller]")]
    [ApiVersion("1.0")]
    [Route("api/v{version:apiVersion}/[controller]")]
    [ApiController]
    public class ChampionsController : ControllerBase
    {

       public ChampionsController(IDataManager Manager) {
            this.ChampionsManager = Manager.ChampionsMgr;
            this.SkinsManager = Manager.SkinsMgr;
       }

        private IChampionsManager ChampionsManager;
        private ISkinsManager SkinsManager;

        // GET api/<ChampionController>/5

        [HttpGet("count")]
        public async Task<IActionResult> GetCount()
        {
            return Ok(ChampionsManager.GetNbItems());
        }

        [HttpGet]
        public async Task<IActionResult> Get(string? name = null,String? skill = null, String? characteristic = null,int index = 0,int size =10)
        {
            if (size - index > 10)
            {
                return BadRequest();
            }
            if (!string.IsNullOrEmpty(name))
            {
                var list = await ChampionsManager.GetItemsByName(name, index, size);
                if (list.Count() != 0)
                {
                    return Ok(list.Select(champion => champion?.ToDTO()));
                }
                else { return NoContent(); }
            }
            else if(!string.IsNullOrEmpty(skill)) {
                var list = await ChampionsManager.GetItemsBySkill(skill, index, size);
                if (list.Count() != 0)
                {
                    return Ok(list.Select(champion => champion?.ToDTO()));
                }
                else { return NoContent(); }
            }
            else if(!string.IsNullOrEmpty (characteristic)) {
                var list = await ChampionsManager.GetItems(index, size);
                if (list.Count() != 0)
                {
                    return Ok(list.Select(champion => champion?.ToDTO()));
                }
                else { return NoContent(); }
            }
            else {
                var list = await ChampionsManager.GetItems(index, size);
                if (list.Count() != 0)
                {
                    return Ok(list.Select(champion => champion?.ToDTO()));
                }
                else { return NoContent(); }
            }
        }

        [HttpGet("name")]
        public async Task<IActionResult> GetByName(String name)
        {
            if (string.IsNullOrEmpty(name)) return BadRequest();
            var list = await ChampionsManager.GetItemsByName(name, 0, 1);
            if (list.Count() == 1)
            {
                return Ok(list.Select(champion => champion?.ToDTO()).First());
            }
            else { return NoContent(); }

        }

        [HttpGet("name/skins")]
        public async Task<IActionResult> GetSkinsByName(String name)
        {
            if (string.IsNullOrEmpty(name)) return BadRequest();
            var list = await ChampionsManager.GetItemsByName(name, 0, 1);
            if (list.Count() == 1)
            {
                var nb = await SkinsManager.GetNbItemsByChampion(list.First());
                if (nb != 0)
                {
                    var skins = await SkinsManager.GetItemsByChampion(list.First(), 0, nb);
                    return Ok(skins.Select(skin => skin?.ToDTO()));
                }
                else { return NoContent(); }
            }
            else { return NoContent(); }
        }

        //[HttpGet("name/skills")]
        //public async Task<IActionResult> GetSkillsByName(String name)
        //{
        //    if (string.IsNullOrEmpty(name)) return BadRequest();
        //    var list = await ChampionsManager.GetItemsByName(name, 0, 1);
        //    if (list.Count() == 1)
        //    {
        //        var skins = await SkinsManager.GetItemsByChampion(list.First(), 0, await SkinsManager.GetNbItemsByChampion(list.First()));
        //        if (skins.Count() != 0)
        //        {
        //            return Ok(skins.Select(skin => skin?.ToDTO()));
        //        }
        //        else { return NoContent(); }
        //    }
        //    else { return NoContent(); }
        //}



        // POST api/<ChampionController>
        [HttpPost]
        public async Task<IActionResult> Post(ChampionDTO champion)
        {
            if (champion == null)
            {
                return UnprocessableEntity();
            }
            else
            {
                var champ = await ChampionsManager.GetItemsByName(champion.Name, 0, 1);
                if(champ.Count() != 0 && champ.FirstOrDefault().Name == champion.Name)
                {
                    return Conflict(champion);
                }
                await ChampionsManager.AddItem(champion.ToChampion());
                return CreatedAtAction("Post",champion);
            
            }
        }

        // PUT api/<ChampionController>/5
        [HttpPut("name")]
        public async Task<IActionResult> Put(string name, ChampionDTO championDTO)
        {
            if(string.IsNullOrEmpty(name))
                return BadRequest();
            if(championDTO == null)
                return UnprocessableEntity();
            var list = await ChampionsManager.GetItemsByName(name, 0, 1);
            if (list.Count() == 1)
            {
                return Ok(ChampionsManager.UpdateItem(list.First(), championDTO.ToChampion()));
            }
            else { return NoContent(); }
        }

        // DELETE api/<ChampionController>/5
        [HttpDelete("name")]
        public async Task<IActionResult> Delete(string name)
        {
            if (string.IsNullOrEmpty(name))
                return BadRequest();
            var list = await ChampionsManager.GetItemsByName(name, 0, 1);
            if(list.Count() == 1){
                return Ok(await ChampionsManager.DeleteItem(list.First()));
            }else { return NoContent(); }
        }
    }
}
